<?php
/**
 * Copyright 2017 by Hannes Peterseim
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Questions, job offers or simply wishes for this product are welcome, please use oxid@petit-souris.de or take a look @ https://gitlab.petit-souris.de for other peaces of software.
 */

/**
 * Class bezahlcodeOxOrder
 */
class bezahlcodeOxOrder extends bezahlcodeOxOrder_parent {

    /**
     * @var null
     */
    private $iban = null;

    /**
     * @var null
     */
    private $bic = null;

    /**
     * @var null
     */
    private $receiver = null;

    /**
     * @var null
     */
    private $amount = null;

    /**
     * @var null
     */
    private $reason = null;

    /**
     * @var null
     */
    private $reasonn = null;

    /**
     * @var null
     */
    private $cur = null;

    /**
     * @var null
     */
    private $reference = null;

    /**
     * @var null
     */
    private $payDate = null;

    /**
     * @var null
     */
    private $integratePayDate = null;

    /**
     * @var null
     */
    private $myShop = null;

    /**
     * @var null
     */
    private $myConfig = null;

    /**
     * @var null
     */
    private $myUser = null;

    /**
     * @var null
     */
    private $fileName = null;

    /**
     * @var int
     */
    private $bcSize = 150;

    /**
     *creating necessary objects
     */
    protected function _baseObjects()
    {
        $this->myUser = oxNew( 'oxuser' );
        $this->myShop = $this->_getActShop();
        $this->myConfig = oxRegistry::getConfig();
        $this->bcSize = $this->myConfig->getConfigParam('bcSizeChoice');
        $this->integratePayDate = $this->myConfig->getConfigParam('integratePayDate');
    }

    /**
     * Injecting picture to the pdf output.
     *
     * @param $oPdf
     */
    protected function _injectBezahlcode ($oPdf) {

        $this->_baseObjects();
        if ($this->oxorder__oxpaymenttype == 'oxidpayadvance' || $this->oxorder__oxpaymenttype == 'oxidinvoice') {
            $this->iban = strip_tags($this->myShop->oxshops__oxibannumber->value);
            $this->bic = strip_tags($this->myShop->oxshops__oxbiccode->value);
            $this->receiver = $this->encodeDtausString($this->myShop->oxshops__oxname->getRawValue());
            $this->amount = $this->encodeAmountString(number_format($this->oxorder__oxtotalbrutsum->value,2,',',''));
            $this->reason = $this->encodeDtausString('Rechnung ' . strip_tags($this->oxorder__oxbillnr->value) . ' Bestellung ' . $this->oxorder__oxordernr->value, 140);
            $this->reasonn = $this->encodeDtausString('Kundenr ' . strip_tags($this->myUser->load($this->oxorder__oxuserid->value)),35);
            $this->cur = $this->getCurrency()->name;
            $this->reference = md5(strip_tags($this->oxorder__oxbillnr->value));
            if ($this->oxorder__oxbilldate->value > '0000-00-00') {
                $this->payDate = strtotime($this->oxorder__oxbilldate->value);
            }
            $oPdf->image($this->generateBezahlCode(), 15, 10, $this->bcSize * 0.2, $this->bcSize * 0.2, '', $oShop->oxshops__oxurl->value);
        }
    }

    /**
     * @param     $aString
     * @param int $aAllowedLength
     *
     * @return string
     * @throws \Exception
     */
    protected function encodeDtausString($aString, $aAllowedLength = 27){
        $aString = strtoupper($aString);
        $aString = preg_replace('|[^0-9A-Z\s\.,&\-+*%/$����]|m', '', $aString);

        if(strlen($aString) <= $aAllowedLength){
            return urlencode($aString);
        } else {
            throw new Exception('Empfängerbezeichnung zu lange.');
        }
    }

    /**
     * @param $aString
     *
     * @return string
     * @throws \Exception
     */
    protected function encodeAmountString($aString){
        // Aufbau: 1-10 Ziffern [,1-2 Ziffern]
        // also 1 oder 1,1 oder 11111,11
        if(preg_match('/^\d{1,10}(?:,\d{1,2})?$/', $aString)){
            return urlencode($aString);
        } else {
            throw new Exception('Betrag ungültig formatiert.');
        }
    }

    /**
     * Generates qr code file.
     *
     * @return string
     */
    protected function generateBezahlCode(){
        try {
            // URL zusammensetzen
            $bezahlCode='bank://singlepayment?';
            $bezahlCode.='name='.$this->receiver;
            $bezahlCode.='&iban='.$this->iban;
            $bezahlCode.='&bic='.$this->bic;
            $bezahlCode.='&amount='.$this->amount;
            $bezahlCode.='&currency='.$this->cur;
            $bezahlCode.='&reason='.$this->reason;
            $bezahlCode.='&reasonN='.$this->reasonn;
            $bezahlCode.='&reference='.$this->reference;
            if ($this->integratePayDate === true && $this->payDate > 0) {
                $bezahlCode.='&executiondate='.$this->payDate;
            }
            $this->fileName = md5($bezahlcode) . '.png';
            include __DIR__ . '/../../services/phpqrcode/qrlib.php';
            // strtoupper wandelt keine Umlaute ausser man ver�ndert die locale
            setlocale (LC_ALL, 'de_DE@euro', 'de_DE', 'de', 'ge');

            // QR-Code erzeugen und ausgeben
            QRcode::pngWithBezahlCode($bezahlCode, $this->myConfig->getOutDir() . $this->fileName, "L", $this->bcSize, 4, false);
            return $this->myConfig->getOutDir() . $this->fileName;

        } catch (Exception $e) {
            echo 'Es ist ein Fehler aufgetreten.',  $e->getMessage(), "\n";
        }
    }


    /**
     * Generates order pdf report file.
     *
     * @param string $sFilename name of report file
     * @param int    $iSelLang  active language
     */
    public function genPdf($sFilename, $iSelLang = 0)
    {
        // setting pdf language
        $this->setSelectedLang($iSelLang);

        $blIsNewOrder = 0;
        // setting invoice number
        if (!$this->oxorder__oxbillnr->value) {
            $this->oxorder__oxbillnr->setValue($this->getNextBillNum());
            $blIsNewOrder = 1;
        }
        // setting invoice date
        if ($this->oxorder__oxbilldate->value == '0000-00-00') {
            $this->oxorder__oxbilldate->setValue(date('Y-m-d', mktime(0, 0, 0, date('m'), date('d'), date('Y'))));
            $blIsNewOrder = 1;
        }
        // saving order if new number or date
        if ($blIsNewOrder) {
            $this->save();
        }

        // initiating pdf engine
        $oPdf = oxNew('oxPDF');
        $oPdf->setPrintHeader(false);
        $oPdf->open();

        // adding header
        $this->pdfHeader($oPdf);

        // adding info data
        switch (oxRegistry::getConfig()->getRequestParameter('pdftype')) {
            case 'dnote':
                $this->exportDeliveryNote($oPdf);
                break;
            default:
                $this->exportStandart($oPdf);
        }

        // adding qr code
        $this->_injectBezahlcode ($oPdf);

        // adding footer
        $this->pdfFooter($oPdf);

        // outputting file to browser
        $oPdf->output($sFilename, 'I');

        // deletes unnecessary qr code file
        unlink($this->myConfig->getOutDir(true) . $this->fileName);

    }

}
